/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.mixin.registry.sync.quilt;

import java.util.Map;
import java.util.stream.Stream;

import org.jetbrains.annotations.NotNull;
import org.quiltmc.qsl.registry.api.event.DynamicRegistryManagerSetupContext;
import org.quiltmc.qsl.registry.api.event.RegistryEntryContext;
import org.quiltmc.qsl.registry.api.event.RegistryEvents;
import org.quiltmc.qsl.registry.impl.DynamicRegistryManagerSetupContextImpl;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import net.fabricmc.fabric.api.event.registry.DynamicRegistryView;
import net.fabricmc.fabric.api.event.registry.RegistryEntryAddedCallback;
import net.fabricmc.fabric.api.event.registry.RegistryEntryRemovedCallback;
import net.minecraft.class_2378;
import net.minecraft.class_2385;
import net.minecraft.class_5321;
import net.minecraft.class_5455;

@Mixin(DynamicRegistryManagerSetupContextImpl.class)
public abstract class DynamicRegistryManagerSetupContextImplMixin implements DynamicRegistryManagerSetupContext, class_5455, DynamicRegistryView {
	@Shadow(remap = false)
	@Final
	private Map<class_5321<?>, class_2385<?>> registries;

	@Shadow
	@Override
	public abstract @NotNull class_5455 registryManager();

	@Shadow
	@Override
	public abstract Stream<class_6892<?>> method_40311();

	@Override
	public class_5455 asDynamicRegistryManager() {
		return this.registryManager();
	}

	@Override
	public Stream<class_2378<?>> stream() {
		return this.registries.values().stream().map(registry -> (class_2378<?>) registry);
	}

	// getOptional is implemented at workaround.DynamicRegistryManagerSetupContextImplMixin

	@Override
	public <T> void registerEntryAdded(class_5321<? extends class_2378<? extends T>> registryRef, RegistryEntryAddedCallback<T> callback) {
		var newCallback = new RegistryEvents.EntryAdded<T>() {
			@Override
			public void onAdded(RegistryEntryContext<T> context) {
				callback.onEntryAdded(context.rawId(), context.id(), context.value());
			}
		};

		this.monitor((class_5321<? extends class_2378<T>>) (Object) registryRef, monitor -> monitor.forAll(newCallback));
	}

	@Override
	public <T> void registerEntryRemoved(class_5321<? extends class_2378<? extends T>> registryRef, RegistryEntryRemovedCallback<T> callback) {
		class_2378<T> registry = (class_2378<T>) this.registries.get(registryRef);

		if (registry != null) {
			RegistryEntryRemovedCallback.event(registry).register(callback);
		}
	}
}
